/////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
// Copyright (c) 2013 Raytrix GmbH. All rights reserved.
/////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
#pragma once

/**
\addtogroup RxCore
**/
/// @{

// ///////////////////////////////////////////////////
// Image Type from Core version 2.8

/////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
/// <summary>	Image data types. </summary>
/////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
#define RX_IMG_BYTE             0x1400
#define RX_IMG_UNSIGNED_BYTE    0x1401
#define RX_IMG_SHORT            0x1402
#define RX_IMG_UNSIGNED_SHORT   0x1403
#define RX_IMG_INT              0x1404
#define RX_IMG_UNSIGNED_INT     0x1405
#define RX_IMG_FLOAT            0x1406
#define RX_IMG_DOUBLE           0x140A

/////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
/// <summary>	Pixel color types. </summary>
/////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
#define RX_IMG_RGB              0x1907
#define RX_IMG_RGBA             0x1908
#define RX_IMG_BGR              0x80E0
#define RX_IMG_BGRA             0x80E1
#define RX_IMG_LUMINANCE        0x1909
#define RX_IMG_LUMINANCE_ALPHA  0x190A

/////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
/// <summary>	Pixel color bayer types </summary>
/////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
#define RX_IMG_BAYER_RG         0x2001
#define RX_IMG_BAYER_BG         0x2002
#define RX_IMG_BAYER_GR         0x2003
#define RX_IMG_BAYER_GB         0x2004

/////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
/// <summary>	Special Data Types </summary>
///
/// In this formats the high and low bits of each pixel are split. In the Following packing description Hx refers to high
/// Bits of a Pixel Value and Lx refers to the low bits of a pixel value with index x. The | represents an 8-bit octet (1 byte)
/////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
#define RX_IMG_10BIT_SPLIT_PACKED       0x1410	// format layout is: H0 | H1 | H2 | H3 | L3 L2 L1 L0 | H4 | H5 | ....
#define RX_IMG_12BIT_SPLIT_PACKED       0x1411	// format layout is: H0 | H1 | L1 L0 | H2 | H3 | L3 L2 | ...

/////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
/// <summary>	Special Pixel Types </summary>
/////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
#define RX_IMG_YUV420P                 0x8757	// this enum belongs to GL_YCBCR_MESA

/////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
// namespace: Rx.Interop.Runtime28
/////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
namespace Rx
{
	namespace Interop
	{
		namespace Runtime28
		{
			/**
			\addtogroup RxCore_Image
			**/
			/// @{

			namespace EDataType
			{
				/////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
				/// <summary>
				/// 	The image data types.
				/// </summary>
				/////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
				enum ID
				{
					Void = 0,

					/************************************************************************/
					/* Non Primitive (custom)                                               */
					/************************************************************************/

					/// <summary> The lower 10 bits of a 16 bit unsigned short are used. The upper 6 bits are unused (zero). </summary>
					Custom_10in16_LSB = 10,
					/// <summary> The upper 10 bits of a 16 bit unsigned short are used. The lower 6 bits are unused (zero). </summary>
					Custom_10in16_MSB,
					/// <summary> The lower 12 bits of a 16 bit unsigned short are used. The upper 4 bits are unused (zero). </summary>
					Custom_12in16_LSB,
					/// <summary> The upper 12 bits of a 16 bit unsigned short are used. The lower 4 bits are unused (zero). </summary>
					Custom_12in16_MSB,

					/// <summary> Each pixel has a size of 10 bit. It should be converted into 16 bit. </summary>
					Custom_Packed10,
					/// <summary> Each pixel has a size of 12 bit. It should be converted into 16 bit. </summary>
					Custom_Packed12,

					SplitPacked10Bits = RX_IMG_10BIT_SPLIT_PACKED,
					SplitPacked12Bits = RX_IMG_12BIT_SPLIT_PACKED,

					/************************************************************************/
					/* Primitive (starting with 0x1400)                                     */
					/************************************************************************/

					/// <summary> 8 bit signed. </summary>
					Byte = RX_IMG_BYTE,
					/// <summary> 8 bit unsigned. </summary>
					UByte = RX_IMG_UNSIGNED_BYTE,
					/// <summary> 16 bit signed. </summary>
					Short = RX_IMG_SHORT,
					/// <summary> 16 bit unsigned. </summary>
					UShort = RX_IMG_UNSIGNED_SHORT,
					/// <summary> 32 bit signed. </summary>
					Int = RX_IMG_INT,
					/// <summary> 32 bit unsigned. </summary>
					UInt = RX_IMG_UNSIGNED_INT,
					/// <summary> Float. </summary>
					Float = RX_IMG_FLOAT,
					/// <summary> Double. </summary>
					Double = RX_IMG_DOUBLE
				};
			}

			namespace EPixelType
			{
				/////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
				/// <summary>
				/// 	The image pixel types..
				/// </summary>
				/////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
				enum ID
				{
					None = 0,
					/// 3 Channels per pixel (Red, Green, Blue)
					RGB = RX_IMG_RGB,
					/// 4 Channels per pixel (Red, Green, Blue, Alpha)
					RGBA = RX_IMG_RGBA,
					/// 3 Channels per pixel (Blue, Green, Red)
					BGR = RX_IMG_BGR,
					/// 4 Channels per pixel (Blue, Green, Red, Alpha)
					BGRA = RX_IMG_BGRA,
					/// 1 Channel per pixel (Luminance)
					Lum = RX_IMG_LUMINANCE,
					/// 2 Channels per pixel (Luminance, Alpha)
					LumA = RX_IMG_LUMINANCE_ALPHA,
					/// 1 Channel per pixel (Bayer pattern RGGB)
					BayerBG = RX_IMG_BAYER_BG,
					/// 1 Channel per pixel (Bayer pattern BGGR)
					BayerGB = RX_IMG_BAYER_GB,
					/// 1 Channel per pixel (Bayer pattern GRBG)
					BayerGR = RX_IMG_BAYER_GR,
					/// 1 Channel per pixel (Bayer pattern GBRG)
					BayerRG = RX_IMG_BAYER_RG,
					/// Planar YUV with 4:2:0 chroma subsampling
					YUV420P = RX_IMG_YUV420P
				};
			}
		}
	}
}

/// @}
