/////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
/// <summary>
/// 	Defines the properties that can be used to configure the camera.
/// 	The following pattern indicates the usage of each property.
///
/// 	[(datatype)(read/write access)(allowed camera state)(camera model)]
///
/// 	- datatype:				shows the data type to use to access the property
/// 	- read/write access:	(R) states a readable property, (W) a writable , (RW) read and write access
/// 	- allowed camera state: indicates at which state a property can be set. (stopped) Property is only available when camera is stopped, (always) Property can be set when camera is running
/// 	- camera model:		    shows if a certain property is available to all models (X) or to a specific (C42) e.g.
/// </summary>
/////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

#ifdef RX_MANAGED_ENUM
	public enum class ECamProperty
#else
	namespace ECamProperty
	{
		enum ID
#endif
{
	/// The default status.
	None = 0,

	//////////////////////////////////////////////////////////////////////////
	// Basic control
	//////////////////////////////////////////////////////////////////////////

	/// Exposure time in seconds [(double)(RW)(always)(x)].
	Exposure,

	/// Framerate in frames per second [(double, int)(RW)(always)(x)].
	Framerate,

	/// Changes the image format [(enum)(RW)(always)(x)].
	ImageFormat,

	/// Set/get current gain factor [(int, double)(RW)(always)(x)].
	Gain,

	/// Changes Focus. Min value is smaller than max value [(int, double)(RW)(always)(R42)].
	Focus,

	/// Selects a video mode such as 1080p [(enum)(RW)(stopped)(R42)]
	VideoMode,

	/// Set/get Automatic gain adjustment [(int)(RW)(always)(x)]
	AutoGain,

	///  Set/get Automatic exposure adjustment  [(int)(RW)(always)(x)]
	AutoExposure,

	/// Set/get the exposure/gain adjustment interval. The AutoExposure/Gain will adjust every N Frames [(int)(RW)(always)(x)]
 	AutoExpUpdateInterval,

	/// Set/get the desired image brightness in a normalized [0,1] interval [(double)(RW)(always)(x)]
 	AutoExpTargetBrightness,

	/// Set/get the desired image brightness tolerance. [(double)(RW)(always)(x)]
	AutoExpTargetBrightnessTolerance,

	/// Set/get the rate of adjustment if the image target brightness is not matched. The value should be in a [0,1] interval. [(double)(RW)(always)(x)]
	AutoExpAdaptionRate,

	//////////////////////////////////////////////////////////////////////////
	// Advanced control
	//////////////////////////////////////////////////////////////////////////

	/// Adjust the pixel transfer speed. [(double)(RW)(stopped)(R42)].
	TransferSpeed = 0x100,

	/// Write 1 to enable the global start for each frame, 0 to disable the global start [(int)(RW)(stopped)(R42)].
	GlobalStart,

	/// Sets the color gain for red. Valid values: 0 to 3.99609375, default = 1 [(double, int)(RW)(always)(R42)].
	ColorGainHwRed,

	/// Sets the color gain for green1. Valid values: 0 to 3.99609375, default = 1 [(double, int)(RW)(always)(R42)].
	ColorGainHwGreen1,

	/// Sets the color gain for green2. Valid values: 0 to 3.99609375, default = 1 [(double, int)(RW)(always)(R42)].
	ColorGainHwGreen2,

	/// Sets the color gain for blue. Valid values: 0 to 3.99609375, default = 1 [(double, int)(RW)(always)(R42)].
	ColorGainHwBlue,

	/// Enables different OIS modes [(int)(RW)(always)(R42)].
	OisMode,

	/// Moves OIS to XY position. Boundaries from -90 to 90 [(int)(RW)(always)(R42)].
	OisPosition,

	/// Open or close shutter [(int)(RW)(always)(R42)].
	Shutter,

	/// Enable or disable the test image generated by the camera hardware [(int)(RW)(always)(x)].
	TestPattern,

	/// Sets or gets the callback mode [(enum)(RW)(stopped)(x)]
	CallbackMode,

	/// Adds another memory pointer to the internal queue for ECallbackMode::ExternalImageBuffer. The size of this
	/// buffer has to be equal or larger than the size returned by ECamProperty::ImageSize. [(void*, char*)(W)(always)(x)]
	AddImageBufferPtr,

	/// Clears all added image buffer [(int)(W)(always)(x)]
	ClearImageBufferPtr,

	/// Gets the width of an image [(int)(R)(stopped)(x)].
	Width,

	/// Gets the height of an image [(int)(R)(stopped)(x)].
	Height,

	/// Gets the height of an image [(int, int)(RW)(stopped)(R42)].
	Resolution,

	/// Gets the ROI start X position [(int)(R)(always)(R42)].
	RoiStartX,

	/// Gets the ROI start Y position [(int)(R)(always)(R42)].
	RoiStartY,

	/// Defines ROI start XY position [(int)(RW)(always)(R42)].
	RoiStart,

	/// Defines horizontal and vertical binning mode. Possible values can be 0, 2, 4, 8 etc. [(int)(RW)(stopped)(R42)].
	Binning,

	/// Defines the Image Transfer Mode. Default Mode is via RAM. More Transfer
	/// Modes are Defined in Rx::RayCam::EImageTransferMode [(enum)(RW)(always)(x)]
	ImageTransferMode,

	/// Returns a pointer to the windows HGLRC struct created by RayCamAPI [(long long)(R)(after open)(x)].
	/// This pointer can be used to create a Shared OpenGL Context.
	OpenGLContextHandle_Windows,

	/// Returns a pointer to the windows HDC struct created by RayCamAPI [(long long)(R)(after open)(x)].
	/// This pointer can be used to create a Shared OpenGL Context.
	DeviceContextHandle_Windows,

	/// Enable/disable fixed pattern noise correction [(int)(RW)(always)(x)]
	FPNEnable,

	/// An enum constant representing the denoise mode option [(enum)(RW)(always)(x)].
	DenoiseMode,

	/// An int constant representing the denoise sigma r option [(double)(RW)(always)(x)].
	DenoiseSigmaR,

	/// An int constant representing the denoise sigma s option [(double)(RW)(always)(x)].
	DenoiseSigmaS,

	/// Enable or disable HDR mode [(int)(RW)(always)(x)]
	HighDynamicRange,

	/// Enable or disable the burst mode. Set value to 5 to get a burst of 5 images [(int)(RW)(always)(x)].
	BurstMode,

	/// Get/Set the SW gamma correction Value [(double)(RW)(always)(x)].
	GammaCorrectionSW,

	/// Get/Set the GPU red gain Value [(double)(RW)(always)(x)].
	ColorGainSWRed,

	/// Get/Set the GPU green gain Value [(double)(RW)(always)(x)].
	ColorGainSWGreen,

	/// Get/Set the GPU blue gain Value [(double)(RW)(always)(x)].
	ColorGainSWBlue,

	// Turn off/on strobe. 0 = strobe off, 1 = single strobe on [d1 and L1 only], 2 = dual strobe on [(int)(RW)(always)(R42)]
	StrobeOn,

	// Represents length to the first pulse. Valid Values: 0 to 3.413ms, 52.083ns step size [(double)(RW)(always)(R42)]
	StrobeLength1,

	// Represents length of the second pulse. Valid Values: 0 to 3.413ms, 52.083ns step size [(double)(RW)(always)(R42)]
	StrobeLength2,

	// Represents delay of the first pulse. Valid values: depends on resolution [(double)(RW)(always)(R42)]
	StrobeDelay1,

	// Represents delay to the second pulse. Valid Values: 0 to 3.413ms, 52.083ns step size [(double)(RW)(always)(R42)]
	StrobeDelay2,

	// Defines the timeout in seconds for the white balancing and devignetting. Default value is 2 seconds. [(double, int)(RW)(always)(x)]
	TimeoutDevignettingWhiteBalance,

	/// Check for integrated optic [(int)(R)(always)(R42)].
	Optics,

	/// Enable Software Sharpening [(int)(RW)(always)(x)].
	SharpeningSW,

	/// Defines internal image buffer size, default is 5 [(int)(RW)(stopped)(Rx)].
	InternalFrameBufferCount,

	/// Defines USB receive buffer multiplier, entered values will be rounded up to the next power of 2, default is 1024. Values below 1024 _can_ cause serious trouble during USB transfer! [(int)(RW)(stopped)(Rx)].
	UsbReceiveBufferMultiplier,

	/// Write 1 to enable the mechanical shutter for each frame, 0 to disable the mechanical shutter. Enable GlobalStart to use this option. [(int)(RW)(stopped)(R42)].
	MechanicalShutter,

	/// The time in s between the trigger signal and frame start event. [(double)(RO)(always)(R42)].
	FrameStartDelay,

	/// Enable unpacking of 10 and 12 bit images to 16 bit
	Unpack1012Bit,

	/// Set the binning mode. The available modes are enumerated in EBinningMode.h  [(int)(RW)(always)(R42)].
	BinningMode,

	/// Sets the zoom factor by changing resolution and binning factors. The output image dimensions will stay the same when zoomed.
	/// Zooming is only available is a video/image configuration with binning is selected. [(int)(RW)(always)(R42)].
	Zoom,

	//////////////////////////////////////////////////////////////////////////
	// Basic information
	//////////////////////////////////////////////////////////////////////////

	/// The camera name [(char*)(R)(always)(x)].
	Name = 0x200,

	/// The unique serial number of the camera [(char*)(R)(always)(x)].
	Serialnumber,

	/// Returns camera type [(int)(R)(always)(x)].
	Type,

	/// A value !=0  represents color [(int)(R)(always)(x)].
	Color,

	/// Gets or sets the bit depth of an image, values can be 8, 10 or 12 [(int)(RW)(always)(R42)].
	Bitdepth,

	/// Representing bayer pattern Rx::RayCam::EBayerPattern::ID [(enum)(R)(always)(x)].
	BayerPattern,

	/// Gets the image size [(int)(R)(always)(x)]
	ImageSize,

	/// Gets the data format of the image [(enum)(R)(always)(x)].
	DataFormat,

	/// Checks, if the camera is still running [(int)(R)(always)(x)].
	IsRunning,

	/// Temperature of the sensor in Celsius [(double)(R)(always)(x)].
	TempSensor,

	/// Temperature of the chip in Celsius [(double)(R)(always)(x)].
	TempChip,

	//////////////////////////////////////////////////////////////////////////
	// Advanced information
	//////////////////////////////////////////////////////////////////////////

	/// Returns the current camera firmware version [(char*)(R)(always)(x)].
	VersionCamFirmware = 0x300,

	/// Only used for HasProperty to check for HW Gain capabilities [(int)(R)(always)(x)].
	ColorGainHW,

	/// Returns true if the system is able to process images on the GPU [(int)(R)(always)(x)].
	GPUCapable,

	/// Returns the number of good frames for the current/last run [(long)(R)(always)(x)].
	GoodFrames,

	/// Returns the number of bad frames for the current/last run [(long)(R)(always)(x)].
	BadFrames,

	/// Returns the number of all frames for the current/last run [(long)(R)(always)(x)].
	TotalFrames,

	/// Returns minimal step size for exposure time changes [(double)(R)(always)(x)].
	ExposureStepSize,

	/// Returns x and y values for the gyroscope [(int, int)(R)(always)(R42)].
	Gyroscope,

	/// Returns x value for the gyroscope [(int)(R)(always)(R42)].
	GyroscopeX,

	/// Returns x value for the gyroscope [(int)(R)(always)(R42)].
	GyroscopeY,
};

#ifndef RX_MANAGED_ENUM
	}	// Namespace
#endif
