#include <iostream>
#include <cstring>
#include <thread>
#include <sstream>

#define RX_STATIC

#include "Rx.RayCam.API/RayCamAPI.h"
#include "Rx.Interop.Runtime31/IException.h"
#include <tiffio.h>


std::string sBasePath;

using namespace std;

////////////////////////////////////////////////////////////////////////////////////////
void HandleException(const Rx::Interop::Runtime31::IException* e)
{
	cout << e->GetMessageText() << " " << e->GetFile() << " " << e->GetLine() << endl;

	if (e->GetInnerException() != nullptr)
	{
		HandleException(e->GetInnerException());
	}
}

////////////////////////////////////////////////////////////////////////////////////////
void raycamErrorCallback(void *POwner, Rx::Interop::Runtime31::IException& ex, Rx::RayCam::IDevice* pxDevice)
{

    cout << "ERROR CALLBACK Received Exception: " << endl;
    HandleException(&ex);
}


// this function will write the data as tiff uncomppressed image to given filename
////////////////////////////////////////////////////////////////////////////////////////
void write_TIFF_file (const char * filename, int w, int h, int iChannels, void* pData)
{
    TIFF* output_image = 0;
    // Open the TIFF file
    if((output_image = TIFFOpen(filename, "w")) == NULL){
          std::cerr << "Unable to write tif file: " << "image.tiff" << std::endl;
    }

    TIFFSetField(output_image, TIFFTAG_IMAGEWIDTH, w);
    TIFFSetField(output_image, TIFFTAG_IMAGELENGTH, h);
    TIFFSetField(output_image, TIFFTAG_BITSPERSAMPLE, 8);
    TIFFSetField(output_image, TIFFTAG_ROWSPERSTRIP, h);
    TIFFSetField(output_image, TIFFTAG_ORIENTATION, (int)ORIENTATION_TOPLEFT);
    TIFFSetField(output_image, TIFFTAG_PLANARCONFIG, PLANARCONFIG_CONTIG);
    TIFFSetField(output_image, TIFFTAG_COMPRESSION, COMPRESSION_NONE);

    if (iChannels == 1)
    {
    	TIFFSetField(output_image, TIFFTAG_SAMPLESPERPIXEL, 1);
    	TIFFSetField(output_image, TIFFTAG_PHOTOMETRIC, PHOTOMETRIC_MINISBLACK);
    }
    else if (iChannels == 3)
    {
    	TIFFSetField(output_image, TIFFTAG_SAMPLESPERPIXEL, 3);
    	TIFFSetField(output_image, TIFFTAG_PHOTOMETRIC, PHOTOMETRIC_RGB);
    }


    // Write the information to the file

    tsize_t image_s;
     if( (image_s = TIFFWriteEncodedStrip(output_image, 0, pData, w*h*iChannels)) == -1)
     {
          std::cerr << "Unable to write tif file: " << filename << std::endl;
     }
     else
     {
          std::cout << "Image is saved! size is : " << image_s << std::endl;
     }

    TIFFWriteDirectory(output_image);
    TIFFClose(output_image);
}



////////////////////////////////////////////////////////////////////////////////////////
void myCallback(void* callbackOwner,
		unsigned int uWidth, unsigned int uHeight, unsigned int uPixelType,
		unsigned int uDataType, void* pData)
{

	static int iImgCount = 0;
	int iChannels = 0;

	cout << "Received Image with size: " << uWidth << "x" << uHeight << endl;

	if (uWidth == 0 || uHeight == 0)
	{
		return;
	}

	if (uPixelType == Rx::Interop::Runtime28::EPixelType::Lum)
	{
		iChannels = 1;
	}
	else if (uPixelType == Rx::Interop::Runtime28::EPixelType::RGB)
	{
		iChannels = 3;
	}

	std::stringstream ss;
        ss << sBasePath << "/image" << iImgCount++ << ".tiff";

	cout << "writing file to: " << ss.str() << " .... " << flush;

        write_TIFF_file(ss.str().c_str(), uWidth, uHeight, iChannels, pData);

	cout << "done" << endl;
}


////////////////////////////////////////////////////////////////////////////////////////
int main(int argc, char **argv)
{
	// Initialize the RayCam Driver and Device.
	// 1. Create and Initialize the Driver
	// 2. Discover the raycam devices
	// 3. Create and Open a device
	// 4. Setup Camera parameter
	// 5. Start Streaming


	Rx::RayCam::IDriver* xMyDriver = 0;
	Rx::RayCam::IDevice* xMyDevice = 0;



	// Set the arguments to default
	int iRepeatCounter = 20;
	int iTriggerIntervallMS = 5000;
	sBasePath = ".";

	// (optional) use the command line arguments
	if (argc >= 2)
	{
		sBasePath = argv[1];
	}

	if (argc >= 3)
	{
		iTriggerIntervallMS = atoi(argv[2]);
	}	

	if (argc >= 4)
	{
		iRepeatCounter = atoi(argv[3]);
	}
	

	try
	{
		cout << "Create Driver" << endl;
		xMyDriver = Rx::RayCam::CreateDriver();


		// Check if the driver creation was successful
		if (xMyDriver == 0)
		{
			cout << "error, driver is null" << endl;
			return 0;
		}

		// Try to init the driver
		cout << "Initialize driver" << endl;
		xMyDriver->InitDriver();

                xMyDriver->RegisterErrorCallback(0, raycamErrorCallback);

		// Discover the devices
		cout << "Discover devices" << endl;
		int iDeviceCnt = 0;
		xMyDriver->DiscoverDevices(iDeviceCnt);

		cout << "found " << iDeviceCnt << " devices" << endl;

		// we do not continue if there is no device avaible
		if (iDeviceCnt == 0)
		{
			cout << "No devices found. Exit!" << endl;
		}

		cout << "Create Device" << endl;
		// Create a IDevice from the first discovered Camera
		xMyDevice = xMyDriver->CreateDevice(0);

		// check for device creation errors
		if (xMyDevice == 0)
		{
			cout << "Error on CreateDevice" << endl;
			return 0;
		}

		cout << "Open Device" << endl;

		// Open the device
		xMyDevice->Open();

		// Register the image callback function. This function is called whenever a new image has been captured
		xMyDevice->RegisterCallback(xMyDevice, myCallback);


		// Set the capture format. Rx::RayCam::EImageFormat::RAW will output an unprocessed image.
		// In order to get a post-processed image (e.g. debayerd) change this value to Rx::RayCam::EImageFormat::RGB.
		xMyDevice->SetProperty(Rx::RayCam::ECamProperty::ImageFormat, Rx::RayCam::EImageFormat::RGB);


		// Reduce the Pixel Transfer speed. This is necessary for USB3 Ports that do not offer the full bandwidth
		// Set this value to 0.0 to have the slowest possible pixel transfer speed.
		// Set this value to 1.0 to archive the highest possilbe pixel transfer speed. If your USB3 Port is not able
		// to handle the data throughput, packages may be lost and the callback will not be called.
		xMyDevice->SetProperty(Rx::RayCam::ECamProperty::TransferSpeed, 0.0);

		// set the resolution
		xMyDevice->SetProperty(Rx::RayCam::ECamProperty::VideoMode, Rx::RayCam::ECamVideoMode::HD_ZOOM1_BIT8);

		double dFpsMin, dFpsMax;
		// Get the min and max framerate for the current resolution and pixelTransferSpeed
		xMyDevice->GetPropertyRange(Rx::RayCam::ECamProperty::Framerate, dFpsMin, dFpsMax);

		// Set the framerate to maximum value
		xMyDevice->SetProperty(Rx::RayCam::ECamProperty::Framerate, dFpsMin);

		// Set the exposure to 0.03 seconds (30 milliseconds)
		xMyDevice->SetProperty(Rx::RayCam::ECamProperty::Exposure, 0.03);

		// Enable the Shutter for single trigger captures
		xMyDevice->SetProperty(Rx::RayCam::ECamProperty::GlobalStart, 1);

		// Set the Trigger mode to single trigger
		xMyDevice->StartStream(Rx::RayCam::ECamTriggerMode::SoftwareSingle);


		// now trigger the camera every iTriggerIntervallMS milliseconds
		// the captured images will be returned via the callback
		cout << "stream started " << endl;
		for (int i = 0; i < iRepeatCounter; i++)
		{
			cout << "Trigger Camera" << endl;
			xMyDevice->Trigger();
			std::this_thread::sleep_for(std::chrono::milliseconds(iTriggerIntervallMS));
		}

		std::this_thread::sleep_for(std::chrono::milliseconds(3600*1000));
		// stop the stream and close the device
		xMyDevice->StopStream();
		xMyDevice->Close();
		xMyDriver->DestroyDevice(xMyDevice);


	}
	catch(Rx::Interop::Runtime31::IException& e)
	{
		HandleException(&e);
		cout << "Exception caught" << endl;
	}


}
