#include <iostream>
#include <cstring>
#include <mutex>

#define RX_STATIC
#include "Rx.RayCam.API/RayCamAPI.h"

#include <GL/glew.h>
#include <GL/glut.h>

using namespace std;

// Store image properties in some global variables
int g_iWidth;
int g_iHeight;
int g_ePixType;
int g_eDatType;
unsigned int g_uDataSize = 0;
unsigned char* g_pData = 0;

// Flag indicating a new image
bool bImageAvailable = false;

// control flags
bool bSingleTrigger = false;
bool bPostProcessing = false;
float fPixelTransferSpeed = 0.5;

// Raycam Device pointer
Rx::RayCam::IDevice* xMyDevice = 0;

// Mutex to secure the image data pointer
std::mutex xMutex;


// glut idle func, only call the display func
void idleFunc(void)
{
	glutPostRedisplay();
}

// Re-Render the image onto a quad
void renderScene(void)
{
	// Store the textureID persistant
	static GLuint iTexID = 0;


	glEnable (GL_TEXTURE_2D);

	// Generate a new texture if there is no valid ID
	if (iTexID == 0)
	{
		glGenTextures(1, &iTexID);
	}

	// Bind the texture
	glBindTexture(GL_TEXTURE_2D, iTexID);

	// Lock the mutex to get exclusiv access to the image data pointer.
	// The callback is called from another thread and does access the image data pointer.
	xMutex.lock();

	// Try to upload the image data to the texture
	if (bImageAvailable && g_pData != 0)
	{

		if (bPostProcessing)
		{
			glTexImage2D(GL_TEXTURE_2D, 0, GL_RGB, g_iWidth, g_iHeight, 0,	GL_RGB, GL_UNSIGNED_BYTE, g_pData);
		}
		else
		{
			glTexImage2D(GL_TEXTURE_2D, 0, GL_RGB, g_iWidth, g_iHeight, 0,	GL_LUMINANCE, GL_UNSIGNED_BYTE, g_pData);
		}

		// Set mandatory texture parameters
		glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_NEAREST);
		glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_NEAREST);
		glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_WRAP_S, GL_CLAMP);
		glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_WRAP_T, GL_CLAMP);

		// Reset the image-available flag
		bImageAvailable = false;

	}

	// Unlock the mutex
	xMutex.unlock();


	// Clear the buffers and redraw the texture (fullscreen quad)
	glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);

	glBegin (GL_QUADS);
	glTexCoord2f(1.0f, 1.0f);
	glVertex3f(1, -1, 0);

	glTexCoord2f(0.0f, 1.0f);
	glVertex3f(-1, -1, 0.0);

	glTexCoord2f(0.0f, 0.0f);
	glVertex3f(-1, 1, 0.0);

	glTexCoord2f(1.0f, 0.0f);
	glVertex3f(1, 1, 0.0);
	glEnd();

	glutSwapBuffers();
}

// glut key function. Set some properties on keypress
void keyFunc(unsigned char key, int x, int y)
{
	double dExp;
	int dFocus;
	int iTriggerMode;

	double fFps;


	double dFramerateMax, dFramerateMin, dExposureMax, dExposureMin;

	switch (key)
	{

	case 'x':
		xMyDevice->GetPropertyRange(Rx::RayCam::ECamProperty::Framerate, dFramerateMin, dFramerateMax);
		xMyDevice->GetProperty(Rx::RayCam::ECamProperty::Framerate, fFps);
		xMyDevice->SetProperty(Rx::RayCam::ECamProperty::Framerate, max(fFps - 0.5,dFramerateMin ));
		break;

	case 'c':
		xMyDevice->GetPropertyRange(Rx::RayCam::ECamProperty::Framerate, dFramerateMin, dFramerateMax);
		xMyDevice->GetProperty(Rx::RayCam::ECamProperty::Framerate, fFps);
		xMyDevice->SetProperty(Rx::RayCam::ECamProperty::Framerate, min(fFps + 0.5,dFramerateMax ));
		break;


	case 'q':
		xMyDevice->StopStream();
		xMyDevice->Close();
		exit(0);
		break;

	case '+':
		xMyDevice->GetProperty(Rx::RayCam::ECamProperty::Exposure, dExp);
		xMyDevice->SetProperty(Rx::RayCam::ECamProperty::Exposure, dExp * 1.1);
		break;

	case '0':
		xMyDevice->GetProperty(Rx::RayCam::ECamProperty::Focus, dFocus);
		xMyDevice->SetProperty(Rx::RayCam::ECamProperty::Focus, dFocus + 15);
		break;

	case '9':
		xMyDevice->GetProperty(Rx::RayCam::ECamProperty::Focus, dFocus);
		xMyDevice->SetProperty(Rx::RayCam::ECamProperty::Focus, dFocus - 15);
		break;

	case '-':
		xMyDevice->GetProperty(Rx::RayCam::ECamProperty::Exposure, dExp);
		xMyDevice->SetProperty(Rx::RayCam::ECamProperty::Exposure, dExp * 0.9);
		break;

	case 't':
		if (bSingleTrigger == false)
		{
			xMyDevice->StopStream();
			xMyDevice->SetProperty(Rx::RayCam::ECamProperty::GlobalStart, 1);
			xMyDevice->StartStream(Rx::RayCam::ECamTriggerMode::SoftwareSingle);
			bSingleTrigger = true;
		}

		xMyDevice->Trigger();
		break;

	case 's':
		if (bSingleTrigger == true)
		{
			xMyDevice->StopStream();
			xMyDevice->SetProperty(Rx::RayCam::ECamProperty::GlobalStart, 0);
			xMyDevice->StartStream(
					Rx::RayCam::ECamTriggerMode::SoftwareContinuous);
			bSingleTrigger = false;
		}
		break;

	case '1':
		xMyDevice->StopStream();

		xMyDevice->SetProperty(Rx::RayCam::ECamProperty::VideoMode, Rx::RayCam::ECamVideoMode::FULLRES_ZOOM1_BIT8);	// Set preferred Video mode

		xMyDevice->GetPropertyRange(Rx::RayCam::ECamProperty::Framerate, dFramerateMin, dFramerateMax);
		xMyDevice->SetProperty(Rx::RayCam::ECamProperty::Framerate, dFramerateMax);	// Set number of frames per second

		xMyDevice->GetPropertyRange(Rx::RayCam::ECamProperty::Exposure, dExposureMin, dExposureMax);
		xMyDevice->SetProperty(Rx::RayCam::ECamProperty::Exposure, dExposureMax);	// Set exposure in seconds

		if (bSingleTrigger)
		{
			xMyDevice->StartStream(Rx::RayCam::ECamTriggerMode::SoftwareSingle);
		}
		else
		{
			xMyDevice->StartStream(Rx::RayCam::ECamTriggerMode::SoftwareContinuous);
		}
		break;

	case '2':
		xMyDevice->StopStream();

		xMyDevice->SetProperty(Rx::RayCam::ECamProperty::VideoMode, Rx::RayCam::ECamVideoMode::UHD_ZOOM1_BIT8);	// Set preferred Video mode

		xMyDevice->GetPropertyRange(Rx::RayCam::ECamProperty::Framerate, dFramerateMin, dFramerateMax);
		xMyDevice->SetProperty(Rx::RayCam::ECamProperty::Framerate, dFramerateMax);	// Set number of frames per second

		xMyDevice->GetPropertyRange(Rx::RayCam::ECamProperty::Exposure, dExposureMin, dExposureMax);
		xMyDevice->SetProperty(Rx::RayCam::ECamProperty::Exposure, dExposureMax);	// Set exposure in seconds

		if (bSingleTrigger)
		{
			xMyDevice->StartStream(Rx::RayCam::ECamTriggerMode::SoftwareSingle);
		}
		else
		{
			xMyDevice->StartStream(Rx::RayCam::ECamTriggerMode::SoftwareContinuous);
		}
		break;

	case '3':
		xMyDevice->StopStream();

		xMyDevice->SetProperty(Rx::RayCam::ECamProperty::VideoMode, Rx::RayCam::ECamVideoMode::HD_ZOOM1_BIT8);	// Set preferred Video mode

		xMyDevice->GetPropertyRange(Rx::RayCam::ECamProperty::Framerate, dFramerateMin, dFramerateMax);
		xMyDevice->SetProperty(Rx::RayCam::ECamProperty::Framerate, dFramerateMax);	// Set number of frames per second

		xMyDevice->GetPropertyRange(Rx::RayCam::ECamProperty::Exposure, dExposureMin, dExposureMax);
		xMyDevice->SetProperty(Rx::RayCam::ECamProperty::Exposure, dExposureMax);	// Set exposure in seconds


		if (bSingleTrigger)
		{
			xMyDevice->StartStream(Rx::RayCam::ECamTriggerMode::SoftwareSingle);
		}
		else
		{
			xMyDevice->StartStream(Rx::RayCam::ECamTriggerMode::SoftwareContinuous);
		}

		break;

	}
}


// The image callback.
void myCallback(void* callbackOwner,
		unsigned int uWidth, unsigned int uHeight, unsigned int iPixelType,
		unsigned int iDataType, void* pData)
{

	cout << "Received Image with size: " << uWidth << "x" << uHeight << endl;

	// Lock the mutex, to get exclusive access to the global image variables
	xMutex.lock();

	g_iWidth = uWidth;
	g_iHeight = uHeight;
	g_eDatType = iDataType;
	g_ePixType = iPixelType;

	unsigned int uSize = uHeight * uWidth;

	// Using post-processing, ww get RGB images, they have 3 times more bytes than luminance images
	if (bPostProcessing)
	{
		uSize *= 3;
	}

	// allocate the memory
	if (g_uDataSize == 0 || g_uDataSize != uSize)
	{
		delete g_pData;
		g_uDataSize = uSize;
		g_pData = new unsigned char[uSize];
	}

	// copy the image data
	memcpy(g_pData, pData, g_uDataSize);
	bImageAvailable = true;

	xMutex.unlock();
}

int main(int argc, char **argv)
{

	for (int i = 1; i < argc; i++)
	{
		if (argv[i][0] == 'p')
		{
			bPostProcessing = true;
		}

		if (argv[i][0] == 't')
		{
			fPixelTransferSpeed = atof(argv[i+1]);
		}
	}

	// glut init
	glutInit(&argc, argv);
	glutInitWindowPosition(10, 10);
	glutInitWindowSize(800, 600);
	glutInitDisplayMode(GLUT_RGBA | GLUT_DOUBLE);
	glutCreateWindow("RayCam Linux example");
	glutDisplayFunc(renderScene);
	glutIdleFunc(idleFunc);
	glutKeyboardFunc(keyFunc);

	cout << "Create Driver" << endl;

	Rx::RayCam::IDriver* xMyDriver = Rx::RayCam::CreateDriver();
	Rx::RayCam::SetInfoLogLevel(99);

	if (xMyDriver == 0)
	{
		cout << "error, driver is null" << endl;
		return 0;
	}

	cout << "Initialize driver" << endl;
	xMyDriver->InitDriver();


	cout << "Discover devices" << endl;
	int iDeviceCnt = 0;
	xMyDriver->DiscoverDevices(iDeviceCnt);

	if (iDeviceCnt == 0)
	{
		cout << "No device found, exit application" << endl;
		return 0;
	}

	cout << "found " << iDeviceCnt << " Devices" << endl;

	cout << "Create Device" << endl;
	xMyDevice = xMyDriver->CreateDevice(0);

	if (xMyDevice == 0)
	{
		cout << "Error on CreateDevice" << endl;
		return 0;
	}

	cout << "Open Device" << endl;

	xMyDevice->Open();

	xMyDevice->RegisterCallback(xMyDevice, myCallback);

	if (bPostProcessing)
	{
		xMyDevice->SetProperty(Rx::RayCam::ECamProperty::ImageFormat, Rx::RayCam::EImageFormat::RGB);
	}
	else
	{
		xMyDevice->SetProperty(Rx::RayCam::ECamProperty::ImageFormat, Rx::RayCam::EImageFormat::RAW);
	}


	xMyDevice->SetProperty(Rx::RayCam::ECamProperty::VideoMode, Rx::RayCam::ECamVideoMode::HD_ZOOM1_BIT8);	// Set preferred Video mode

	double dFramerateMax, dFramerateMin, dExposureMax, dExposureMin;
	xMyDevice->GetPropertyRange(Rx::RayCam::ECamProperty::Framerate, dFramerateMin, dFramerateMax);
	xMyDevice->SetProperty(Rx::RayCam::ECamProperty::Framerate,dFramerateMax);	// Set number of frames per second

	xMyDevice->GetPropertyRange(Rx::RayCam::ECamProperty::Exposure, dExposureMin, dExposureMax);
	xMyDevice->SetProperty(Rx::RayCam::ECamProperty::Exposure, min(0.033,dExposureMax));	// Set exposure in seconds

	xMyDevice->SetProperty(Rx::RayCam::ECamProperty::AutoGain, 1);
	xMyDevice->SetProperty(Rx::RayCam::ECamProperty::AutoExposure, 1);

	xMyDevice->SetProperty(Rx::RayCam::ECamProperty::TransferSpeed, fPixelTransferSpeed);

	xMyDevice->StartStream(Rx::RayCam::ECamTriggerMode::SoftwareContinuous);

	glutMainLoop();

	xMyDevice->StopStream();

	xMyDevice->Close();

	xMyDriver->DestroyDevice(xMyDevice);

}

